*--------------------------------------------------------------------------------------------------
* File: 	C:\Projects\Stata Weighting Book\program files\Program files\3 Nonresponse\Ex.3.5_cforest.do
* Project:  Stata weighting book
* Date: 	02/21/2016
* Revised:
* Author:	R. Valliant
* Purpose:	Illustrate use of R party package, cforest function. Call R from Stata.
*--------------------------------------------------------------------------------------------------

log using "C:\Projects\Stata Weighting Book\Program files\3 Nonresponse\Ex.3.5_cforest.smcl", replace

* Install Stata package rsource from http://fmwww.bc.edu/RePEc/bocode/r if not previously installed 
* set globals so that Stata finds installed version of R
 global Rterm_path `"C:\Program Files\R\R-3.3.2\bin\x64\Rterm.exe"'
 global Rterm_options `"--vanilla"'

rsource, terminator(END_OF_R)
    # How does cforest do on with training vs. holdout samples
    # sample 80% as training sample
   require(PracTools)
   require(party)

   set.seed(1585951449)
   data(nhis)
   sam <- sample(1:nrow(nhis), 0.8*nrow(nhis))    
   nhis.train <- nhis[sam,]
   dim(nhis.train)
   #[1] 3128   15      same size as training dsn for Schonlau's boost

   nhis.test <- nhis[-sam,]

   crf.nhis1 <- cforest(as.factor(resp) ~ age + as.factor(hisp) +
                           as.factor(race) +
                           as.factor(parents_r) + as.factor(educ_r),
                       control = cforest_control(ntree = 500, mincriterion = 0, trace = TRUE), data=nhis.train)

   crfnhis.prob1 <- predict(crf.nhis1,newdata=nhis.test,type="prob")
   crf.prob1 <- matrix(0, nrow=nrow(nhis.test), ncol=2)
   for (i in 1:length(crfnhis.prob1)) {
     crf.prob1[i,] = crfnhis.prob1[[i]]
   }
   apply(crf.prob1,2,mean)
   #[1] 0.3070593 0.6929407

       # majority vote classification
   cl.test <- crf.prob1[,2] >= 0.5
   table(nhis.test$resp, cl.test)
#       FALSE TRUE
#     0    32  199
#     1    40  512

   require(lsr)
   cramersV(nhis.test$resp, cl.test)
   #[1] 0.09942009     it was 0.0714 for Schonlau's boost
END_OF_R


*--------------------------------------------------------------------------------------------------
*   Stata code

use "C:\\Projects\\Stata Weighting Book\\Data\\nhis\\crfprobs.dta", clear
merge 1:1 ID using "C:\Projects\Stata Weighting Book\Data\nhis\logistpreds.dta"
twoway scatter logistpreds crf_p1 || line crf_p1 crf_p1, legend(off) ytitle("Logistic predictions") xtitle("cforest predictions")
    
pwcorr logistpreds crf_p1     
graph export "C:\Projects\Stata Weighting Book\Tex files\Figures\Fig32logistcforestTEST.pdf", replace

use "C:\Projects\Stata Weighting Book\Data\nhis\nhis.dta", clear
generate random = runiform()
sort random
gen intest = _n <= 783
gen intrain = _n > 783

    * save test data set
preserve
keep if intest == 1
*save "C:\Projects\Stata Weighting Book\Data\nhis\nhistest.dta", replace
restore

    * extract training dataset
keep if intrain == 1
logit resp age i.hisp i.race i.parents_r i.educ_r
predict predp, pr
summarize predp
*--------------------------------------------------------------------------------------------------

    * Put predicted probs into classes and do boxplot
    * group cforest preds into 5 classes
use "C:\\Projects\\Stata Weighting Book\\Data\\nhis\\crfprobs.dta", clear    
sort crf_p1
pctile qcrfpreds = crf_p1, nq(5) genp(crfperc)
list crfperc qcrfpreds in 1/5
tab qcrfpreds
*percentiles |
*  of crf_p1 |      Freq.     Percent        Cum.
*------------+-----------------------------------
*   .5731861 |          1       25.00       25.00
*   .6563813 |          1       25.00       50.00
*   .7365259 |          1       25.00       75.00
*   .8151672 |          1       25.00      100.00
*------------+-----------------------------------
*      Total |          4      100.00

egen pclass=cut(crf_p1), at(0, 0.5731861, 0.6563813, 0.7365259, 0.8151672, 1.0)
tab pclass
*     pclass |      Freq.     Percent        Cum.
*------------+-----------------------------------
*          0 |        785       20.07       20.07
*   .5731861 |        779       19.92       39.99
*   .6563813 |        799       20.43       60.42
*   .7365259 |        743       19.00       79.42
*   .8151672 |        805       20.58      100.00
*------------+-----------------------------------
*      Total |      3,911      100.00

egen mcrf = mean(crf_p1), by(pclass) 
tab mcrf
*
*       mcrf |      Freq.     Percent        Cum.
*------------+-----------------------------------
*   .4894243 |        785       20.07       20.07
*   .6194608 |        779       19.92       39.99
*   .6951861 |        799       20.43       60.42
*   .7755473 |        743       19.00       79.42
*   .8802271 |        805       20.58      100.00
* ------------+-----------------------------------
*      Total |      3,911      100.00

recode pclass (0 = 0.489) (0.5731861 = 0.619) (0.6563813 = 0.695) (0.7365259 = 0.776) (0.8151672 = 0.880)

graph box crf_p1, over(pclass) ylabel(0.25(0.05)1.0) ytitle("cforest predictions") saving(crfbox, replace)

log close
